package core;
import geom.Arc;
import geom.DPoint;
import geom.Point;
import gui.MainFrame;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Locale;
import java.util.Observable;

import javax.vecmath.Vector3d;

import drawing.Enviroment;
import drawing.PaintBox;

import shapes.Pattern;
import shapes.Road;


public class Core extends Observable {
	
	public static Core instance;
	
	public PaintBox paintBox;
	public MainFrame mainFrame;
	public Road selectedRoad = null;
	public Pattern selectedPattern = null;

	public boolean isPointEditing = false;
	public boolean isRoadEditing = true;
	
	public Session session = new Session();
	
	public void newRoad(String name) {
		
		paintBox.getShapes().remove(selectedRoad);
		selectedRoad = new Road(Enviroment.ROAD_COLOR, name);
		
		this.session.roads.add(selectedRoad);
		this.paintBox.addShape(selectedRoad);
		
		this.paintBox.repaint();
	}
	
	public void deleteRoad(int roadIndex) {
		
		if ( roadIndex >= 0 && !checkPatternForRoad(this.session.roads.get(roadIndex))) {

			this.session.roads.remove(roadIndex);
			this.paintBox.getShapes().remove(selectedRoad);

			selectedRoad = null;

			this.paintBox.repaint();
		}
	}
	
	public void switchRoad(int roadIndex) {
		
		if ( roadIndex >= 0 ) {
			
			this.paintBox.getShapes().remove(selectedRoad);
			selectedRoad = this.session.roads.get(roadIndex);
			this.paintBox.addShape(selectedRoad);
			
			if ( selectedPattern != null && selectedPattern.road != selectedRoad && !isRoadEditing) {
				
				selectedPattern.road = selectedRoad;
				restartCarPosition();
			}
		}
		else {
			
			paintBox.getShapes().remove(selectedRoad);
			selectedRoad = null;
		}
		
		this.paintBox.repaint();
	}
	
	public void addRoadPoint(Vector3d position) {
		
		if ( selectedRoad != null ) {
			
			DPoint newPoint = null;
			
			if ( this.selectedRoad.getPoints().size() > 0 ) {
				
				Vector3d deriv = new Vector3d(position);
				deriv.sub(this.selectedRoad.getPoints().get(this.selectedRoad.getPoints().size() - 1).position);
				
				deriv.normalize();
				deriv.scale(5);
				deriv.add(position);
				
				newPoint = new DPoint(position, new Point(deriv));
			}
			else {
				
				newPoint = new DPoint(position);
			}
			
			selectedRoad.addPoint(newPoint);
		}
		
		this.paintBox.repaint();
	}
	
	public void deleteRoadPoint() {
		
		if ( selectedRoad != null ) {
			
			selectedRoad.removeLastPoint();
			
			paintBox.repaint();
		}
	}
	
	public void restartCarPosition() {
		
		double x0 = this.selectedRoad.points.get(0).position.x;
		double y0 = this.selectedRoad.points.get(0).position.y;

		double x1 = this.selectedRoad.points.get(1).position.x;
		double y1 = this.selectedRoad.points.get(1).position.y;
			
		Vector3d velocity = new Vector3d(x1 - x0, y1 - y0, 0);
		velocity.normalize();
		velocity.scale(Enviroment.DEFAULT_VELOCITY);
		
		this.selectedPattern.setVelocity(velocity);
		this.selectedPattern.setPosition(new Vector3d(x0, y0, 0));
	}

	public boolean newPattern() {
		
		if ( this.selectedRoad != null && this.selectedRoad.points.size() > 1) {

			double x0 = this.selectedRoad.points.get(0).position.x;
			double y0 = this.selectedRoad.points.get(0).position.y;

			double x1 = this.selectedRoad.points.get(1).position.x;
			double y1 = this.selectedRoad.points.get(1).position.y;
			
			Vector3d velocity = new Vector3d(x1 - x0, y1 - y0, 0);
			velocity.normalize();
			velocity.scale(Enviroment.DEFAULT_VELOCITY);

			paintBox.getShapes().remove(selectedPattern);
			
			if ( selectedPattern != null) {
				
				Pattern newPattern = new Pattern(this.selectedRoad, selectedPattern.getPosition().x, 
						selectedPattern.getPosition().y, selectedPattern.getVelocity().x, selectedPattern.getVelocity().y);
				
				newPattern.steer = selectedPattern.steer;
				newPattern.throttle = selectedPattern.throttle;
				
				selectedPattern = newPattern;
			}
			else {
				selectedPattern = new Pattern(this.selectedRoad, x0, y0, velocity.x, velocity.y);
			}
			selectedPattern.setZIndex(10);

			this.session.patterns.add(selectedPattern);
			this.paintBox.addShape(selectedPattern);

			this.paintBox.repaint();
			
			return true;
		}
		else {
			return false;
		}
	}
	
	public void movePattern(int about) {
		
		int newIndex = this.session.patterns.indexOf(this.selectedPattern) + about;
		
		if ( newIndex >= 0 ) {
			
			this.session.patterns.remove(this.selectedPattern);
			this.session.patterns.add(newIndex, this.selectedPattern);
		}
	}
	
	public void switchPattern(int patternIndex) {
		
		if ( patternIndex >= 0 ) {
			
			this.paintBox.getShapes().remove(selectedPattern);
			selectedPattern = this.session.patterns.get(patternIndex);
			this.paintBox.addShape(selectedPattern);
			
			switchRoad(this.session.roads.indexOf(this.selectedPattern.road));
		}
		else {
			
			paintBox.getShapes().remove(selectedPattern);
			selectedPattern = null;
		}
		
		this.paintBox.repaint();
	}
	
	public void deletePattern(int patternIndex) {
		
		this.session.patterns.remove(patternIndex);
		this.paintBox.getShapes().remove(selectedPattern);
		
		selectedPattern = null;
		
		this.paintBox.repaint();
	}

	public Core(PaintBox paintBox, MainFrame mainFrame) {
		super();
		this.paintBox = paintBox;
		this.mainFrame = mainFrame;
		
		Core.instance = this;
	}
	
	public boolean checkPatternForRoad(Road road) {
		
		for (Pattern pattern : this.session.patterns) {
			
			if ( pattern.road == road ) return true;
		}
		
		return false;
	}
	
	public void recomputePatterns() {
	
		for (Pattern pattern : this.session.patterns) {
			
			pattern.update(null, null);
		}
	}
	
	public void saveSession(File file) {

		try {
			
			ObjectOutputStream out = new ObjectOutputStream(new FileOutputStream(file));
			out.writeObject(this.session);
			out.close();
			
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public void loadSession(File file) {
		
		try {
			
			ObjectInputStream in = new ObjectInputStream(new FileInputStream(file));	
			this.session = (Session)in.readObject();		
			in.close();
			
			session.attachObservers();
			
			this.mainFrame.updateListData();
			
			if ( this.session.patterns.size() > 0) {
				
				this.switchPattern(0);
			}
			else {
				this.switchRoad(-1);
				this.switchPattern(-1);
			}
			
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void exportSession(File file, boolean generateMirror, boolean randomizePatterns) {
		
		DecimalFormat decFormat = new DecimalFormat("#0.0000", new DecimalFormatSymbols(Locale.US));
		
		try {
			
			FileWriter writer = new FileWriter(file);

			double maxDistance = Double.MIN_VALUE;
			double maxVelocity = Double.MIN_VALUE;
			double maxAngle = Double.MIN_VALUE;
			
			for (Pattern pattern : this.session.patterns) {
				
				if ( Math.abs(pattern.distance) > maxDistance ) maxDistance = Math.abs(pattern.distance);
				if ( pattern.getVelocity().length() > maxVelocity) maxVelocity = pattern.getVelocity().length();
				
				for (Arc arc : pattern.arcs) {
					
					if ( Math.abs(arc.arcAngle) > maxAngle ) {
						
						maxAngle = Math.abs(arc.arcAngle);
					}
				}
			}
			
			maxAngle = maxAngle / Math.PI * 180;
			
			writer.write("#TrainSet Generator output\n\n");
			writer.write("#inputs description <name> <min> <max>\n");
			writer.write("#number of inputs\n");
			writer.write("7\n");
			writer.write("#km/h\n");
			writer.write("velocity 0 " + decFormat.format(Utils.msToKmh(maxVelocity)) + "\n");
			writer.write("#m\n");
			writer.write("distance " + decFormat.format(-maxDistance) + " " + decFormat.format(maxDistance) + "\n");
			writer.write("#deg\n");
			
			for (int i = 0; i < 5; i++) {
				
				writer.write("fi" + i + " " + decFormat.format(-maxAngle) + " " + decFormat.format(maxAngle) +"\n");
			}
			
			writer.write("\n#outputs description <name> <min> <max>\n");
			writer.write("#number of outputs\n");
			writer.write("2\n");
			writer.write("throttle 0 1\n");
			writer.write("steer 0 1\n\n");
			
			writer.write("#patterns <velocity> <distance> <fi0>..<fi4> | <throttle> <steer>\n");
			writer.write("#number of patterns\n");
			
			int numOfPatterns = this.session.patterns.size();
			
			if ( generateMirror ) {
				
				numOfPatterns *= 2;
			}
			writer.write( numOfPatterns + "\n");
			
			ArrayList<String> exportList = new ArrayList<String>();
			
			for (Pattern pattern : this.session.patterns) {
				
				String patternString = decFormat.format(Utils.msToKmh(pattern.getVelocity().length())) + " ";
				
				patternString += decFormat.format(pattern.distance) + " ";
				
				for (int i = 0; i < 5; i++) {
					
					patternString += decFormat.format(pattern.arcs.get(i).arcAngle / Math.PI * 180) + " ";
				}
				
				patternString += "| " + decFormat.format(pattern.throttle) + " " + decFormat.format(pattern.steer);
				
				exportList.add(patternString);
				
				if ( generateMirror ) {

					patternString = decFormat.format(Utils.msToKmh(pattern.getVelocity().length())) + " ";
					patternString += decFormat.format(-pattern.distance) + " ";

					for (int i = 0; i < 5; i++) {
						patternString += decFormat.format(-pattern.arcs.get(i).arcAngle / Math.PI * 180) + " ";
					}

					patternString += "| " + decFormat.format(pattern.throttle) + " " + decFormat.format(1 - pattern.steer);
					
					exportList.add(patternString);
				}
			}
			
			if ( randomizePatterns ) {
				
				Collections.shuffle(exportList);
			}
			
			for (String patternString : exportList) {
				
				writer.write(patternString + "\n");
			}
			
			writer.close();
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}
