package drawing;
import geom.Point;
import geom.Shape;
import geom.ShapeComparator;

import java.awt.Color;
import java.awt.Graphics;

import java.util.*;

import javax.swing.JPanel;
import javax.vecmath.Matrix4d;

import core.Core;

public class PaintBox extends JPanel {
	
	private double pixelsPerUnit;
	private double zoom = 1;
	
	private Matrix4d windowTransform;
	private ArrayList<Shape> shapes;

    public void paint(Graphics g) {
    	
    	super.paint(g);
    	this.setBackground(Color.WHITE);
    	drawContent(g);
    }

	public PaintBox(double pixelsPerUnit) {
		
		super();
		
		this.pixelsPerUnit = pixelsPerUnit;
		
		this.windowTransform = new Matrix4d();
		this.windowTransform.setIdentity();
		
		this.shapes = new ArrayList<Shape>();
	}
	
	public void updateTransform() {
		
		this.windowTransform.setIdentity();
		
		windowTransform.m00 =  pixelsPerUnit * zoom; 
		windowTransform.m11 = -pixelsPerUnit * zoom;
		windowTransform.m03 = 10;
		windowTransform.m13 = this.getHeight() - 10;
	}
	
	public synchronized void drawContent(Graphics g) {
		
		//draw grid
		
		int width = this.getWidth();
		int height = this.getHeight();
		
		int scale = zoom < 0.5 ? 10 : 5;
		double step = Enviroment.PIXELS_PER_UNIT * scale * zoom;
		
		int counter = 0;
		int size = width > height ? width : height;
		
		g.setColor(new Color(0.6f, 1, 0.6f));
		
		while ( counter < size ) {
			
			g.drawLine(0, height - counter - 10 , width, height - counter - 10);
			g.drawLine(counter + 10 , 0, counter + 10, height);
			
			counter += (int)step;
		}
		
		Point p0 = new Point(0,0);
		Point p1 = new Point(scale,0);
		
		p0.transform(windowTransform);
		p1.transform(windowTransform);
		
		g.setColor(Color.RED);
		g.drawLine(p0.finalX, p0.finalY, p1.finalX, p1.finalY);
		g.drawLine(p0.finalX, p0.finalY - 5, p0.finalX, p0.finalY + 5);
		g.drawLine(p1.finalX, p1.finalY - 5, p1.finalX, p1.finalY + 5);
		
		g.drawString(scale + " m", (p0.finalX + p1.finalX) / 2 - 10, p0.finalY - 4);
		
		for (Drawable shape : this.shapes) {
			
			shape.transform(windowTransform);
			shape.draw(g);
		}
	}
	
	public void addShape(Shape shape) {
		
		this.shapes.add(shape);
		
		 Shape[] array = this.shapes.toArray(new Shape[1]);
		 Arrays.sort(array, new ShapeComparator());
		 
		 this.shapes.clear();
		 
		 for (int i = 0; i < array.length; i++) {
			
			 this.shapes.add(array[i]);
		}
	}

	public Matrix4d getWindowTransform() {
		return windowTransform;
	}

	public void setWindowTransform(Matrix4d windowTransform) {
		this.windowTransform = windowTransform;
	}

	public ArrayList<Shape> getShapes() {
		return shapes;
	}

	public double getZoom() {
		return zoom;
	}

	public void setZoom(double zoom) {
		this.zoom = zoom;
		updateTransform();
	}
}