package geom;

import java.awt.Color;
import java.awt.Graphics;

import javax.vecmath.Matrix4d;
import javax.vecmath.Vector3d;

import drawing.Enviroment;

public class Arc extends Shape {
	
	public String name;
	public DPoint dp0;
	public DPoint dp1;
	
	public double startAngle = 0;
	public double arcAngle = 0;
	
	public Arc(Color color, String name, Vector3d position, Vector3d d0, Vector3d d1) {
		
		super(color);
		this.name = name;
		
		Vector3d p0 = new Vector3d(d0);
		p0.scale(Enviroment.DERIV_MUL);
		p0.add(position);
		dp0 = new DPoint(position, new Point(p0));
		dp0.setColor(color);
		dp0.derivLine.setColor(color);
		dp0.showCotrolPoints = false;
		
		Vector3d p1 = new Vector3d(d1);
		p1.scale(Enviroment.DERIV_MUL);
		p1.add(position);
		dp1 = new DPoint(position, new Point(p1));
		dp1.setColor(color);
		dp1.derivLine.setColor(color);
		dp1.showCotrolPoints = false;
	}
	
	public void transform(Matrix4d windowTransform) {
		
		dp0.transform(windowTransform);
		dp1.transform(windowTransform);
	}

	@Override
	public void draw(Graphics g) {
		
		if ( this.arcAngle > 0 ) {
			
			g.setColor(Color.RED);
		}
		else {
			
			g.setColor(Color.GREEN);
		}	
		
		int start = (int)Math.round(startAngle / Math.PI * 180);
		int angle = (int)Math.round(arcAngle / Math.PI * 180);
		
		if ( angle != 0) {
		
			g.fillArc(dp0.finalX - 30, dp0.finalY - 30, 60, 60, start, angle);
		}

		super.draw(g);
		dp0.draw(g);
		dp1.draw(g);
	}

	@Override
	public String toString() {
		
		return this.name + ": " + Enviroment.NUM_FORMAT.format(this.arcAngle /Math.PI * 180) + "";
	}
}
