package geom;

import java.awt.Color;
import java.util.ArrayList;

import javax.vecmath.Vector3d;

public class Bezier extends Curve {
	
	public Bezier(Color color, DPoint start, DPoint end, int subDiv) {
		super(color, start, end, subDiv);
	}
	
	@Override
	public ArrayList<Vector3d> generateTangents() {
		
		ArrayList<Vector3d> outputPoints = new ArrayList<Vector3d>();
		
		for (int i = 0; i < subDiv; i++) {
			
			double t = (double)i / subDiv;
			double _t = 1 - t;
			double t2 = t*t;
			double _t2 = _t*_t;
			
			double F0 = -3*_t2;
			double F1 = 3*_t2 - 6*t*_t;
			double F2 = 6*t*_t - 3*t2;
			double F3 = 3*t2;
			
			Vector3d X = new Vector3d();
			
			Vector3d tmp = new Vector3d(start.position);
			tmp.scale(F0);
			X.add(tmp);
			
			tmp.set(start.deriv.position);
			tmp.scale(F1);
			X.add(tmp);
			
			tmp.set(end.deriv2.position);
			tmp.scale(F2);
			X.add(tmp);
			
			tmp.set(end.position);
			tmp.scale(F3);
			X.add(tmp);
			
			X.normalize();
			
			outputPoints.add(X);
		}
		
		Vector3d last = end.getDerivative();
		last.normalize();
		
		outputPoints.add(last);
		
		return outputPoints;
	}

	@Override
	public ArrayList<Point> generatePoints() {

		ArrayList<Point> outputPoints = new ArrayList<Point>();
		
		for (int i = 0; i < subDiv; i++) {
			
			double t = (double)i / subDiv;
			double t3 = t*t*t;
			double t2 = t*t;
			double _t = 1 - t;
			
			double F0 = _t*_t*_t;
			double F1 = 3*t*_t*_t;
			double F2 = 3*t2*_t;
			double F3 = t3;
			
			Vector3d X = new Vector3d();
			
			Vector3d tmp = new Vector3d(start.position);
			tmp.scale(F0);
			X.add(tmp);
			
			tmp.set(start.deriv.position);
			tmp.scale(F1);
			X.add(tmp);
			
			tmp.set(end.deriv2.position);
			tmp.scale(F2);
			X.add(tmp);
			
			tmp.set(end.position);
			tmp.scale(F3);
			X.add(tmp);
			
			outputPoints.add(new Point(X));
		}
		
		outputPoints.add(end);
		
		return outputPoints;
	}
}
