package geom;

import java.awt.Graphics;
import java.util.Observable;
import java.util.Observer;

import javax.vecmath.Matrix4d;
import javax.vecmath.Vector3d;

import drawing.Catchable;

public class DPoint extends Point implements Observer {

	public Point deriv;
	public Point deriv2;
	public boolean showCotrolPoints = true;
	
	public Line derivLine;
	
	private void init() {
		
		this.deriv2 = new Point(0,0);	
		
		attachObservers();
		this.derivLine = new Line(deriv2, deriv);
	}

	public DPoint(Vector3d position, Point deriv) {
		super(position);
		this.deriv = deriv;
		init();
	}

	public DPoint(double x, double y, double z, Point deriv) {
		super(x, y, z);
		this.deriv = deriv;
		init();
	}

	public DPoint(double x, double y, Point deriv) {
		super(x, y);
		this.deriv = deriv;
		init();
	}

	public DPoint(Vector3d position) {
		super(position);
		this.deriv = new Point(position.x, position.y + 5, position.z);
		init();
	}

	@Override
	public void draw(Graphics g) {
		
		derivLine.draw(g);
		super.draw(g);
		
		if ( showCotrolPoints ) {
			
			deriv.draw(g);
			deriv2.draw(g);
		}
	}

	@Override
	public Catchable isCatch(double x, double y) {

		if ( deriv.isCatch(x, y) != null ) {
			
			return deriv;
		}
		else if ( deriv2.isCatch(x, y) != null ) {
			
			return deriv2;
		}
		else {
		
			return super.isCatch(x, y);
		}
	}

	@Override
	public void move(double x, double y) {
		// TODO Auto-generated method stub
		
		deriv.position.x += x;
		deriv.position.y += y;
		
		deriv2.position.x += x;
		deriv2.position.y += y;
		
		super.move(x, y);
	}

	@Override
	public void transform(Matrix4d windowTransform) {
		
		deriv.transform(windowTransform);
		deriv2.transform(windowTransform);
		super.transform(windowTransform);
	}
	
	public Vector3d getDerivative() {
		
		Vector3d derivative = new Vector3d(this.deriv.position);
		derivative.sub(this.position);
		
		return derivative;
	}
	
	public Vector3d getDerivative2() {
		
		Vector3d derivative = new Vector3d(this.deriv2.position);
		derivative.sub(this.position);
		
		return derivative;
	}

	public void update(Observable o, Object arg) {
		
		if ( o == deriv ) {
			
			deriv2.position.set(this.position);
			deriv2.position.sub(getDerivative());
		}
		else if ( o == deriv2 ) {
			
			deriv.position.set(this.position);
			deriv.position.sub(getDerivative2());
		}
		
		setChanged();
		notifyObservers();
	}

	@Override
	public void attachObservers() {
		
		deriv.addObserver(this);
		deriv2.addObserver(this);
		
		this.update(deriv, null);
	}
}
