package geom;
import java.awt.Color;
import java.awt.Graphics;

import javax.vecmath.Matrix4d;
import javax.vecmath.Vector3d;
import javax.vecmath.Vector4d;

import drawing.Catchable;
import drawing.Enviroment;
import drawing.Moveable;

public class Point extends Shape implements Catchable, Moveable {
	
	public Vector3d position = new Vector3d(0,0,0);
	
	public int finalX;
	public int finalY;
	
	public Point(Vector3d position) {
		
		this.position.set(position);
	}
	
	public Point(double x, double y, double z) {
		
		this.position.x = x;
		this.position.y = y;
		this.position.z = z;
	}
	
	public Point(double x, double y) {
	
		this(x, y, 0);
	}

	public void transform(Matrix4d windowTransform) {
		
		Vector4d transformed = new Vector4d();
		
		windowTransform.transform(new Vector4d(this.position.x, this.position.y, this.position.z, 1), transformed);
		
		this.finalX = (int)Math.round(transformed.x / transformed.w);
		this.finalY = (int)Math.round(transformed.y / transformed.w);
	}

	public void draw(Graphics g) {
		
		super.draw(g);
		g.drawRect(this.finalX - 3, this.finalY - 3, 6, 6);
	}

	public void move(double x, double y) {
		
		this.position.x += x;
		this.position.y += y;
		
		setChanged();
		notifyObservers();
	}
	
	public void setPosition(Vector3d position) {
		
		this.position.set(position);
		
		setChanged();
		notifyObservers();
		
		afterMove();
	}
	
	public void afterMove() {
		
		setChanged();
		notifyObservers(this);
	}

	public Catchable isCatch(double x, double y) {
		
		double delta = 8 / Enviroment.PIXELS_PER_UNIT;
		
		if ( Math.abs(position.x - x) < delta && Math.abs(position.y - y) < delta) return this;
		else return null;
	}
}
