package geom;
import java.awt.Color;
import java.awt.Graphics;
import java.util.ArrayList;

import javax.vecmath.Matrix4d;
import javax.vecmath.Vector3d;

public class PolyLine extends Shape {
	
	protected ArrayList<Point> points = new ArrayList<Point>();
	boolean showPoints = false;

	public Color fillColor;
	public boolean fillPolygon = false;
	
	public PolyLine(Color color, ArrayList<Point> points, boolean showPoints) {
		
		super(color);
		this.showPoints = showPoints;
		
		this.points = points;	
	}
	
	public void addPoint(Point p) {
		
		this.points.add(p);
	}
	
	public void transform(Matrix4d windowTransform) {
	
		for (Point point : this.points) {
			
			point.transform(windowTransform);
		}
	}

	public void draw(Graphics g) {

		int[] xPoint = new int[this.points.size() + 1];
		int[] yPoint = new int[this.points.size() + 1];	
		
		int i = 0;
		
		for (Point point : this.points) {
			
			if ( this.showPoints ) {
				
				point.draw(g);
			}
			
			xPoint[i] = point.finalX;
			yPoint[i] = point.finalY;
			
			i++;
		}
		
		xPoint[i] = xPoint[0];
		yPoint[i] = yPoint[0];
		
		
		if ( fillPolygon ) {
			
			g.setColor(this.fillColor);
			g.fillPolygon(xPoint, yPoint, this.points.size());
			
			super.draw(g);
			g.drawPolyline(xPoint, yPoint, this.points.size() + 1);
		}
		else {
			
			//super.draw(g);
			g.drawPolyline(xPoint, yPoint, this.points.size());
		}
	}

	public boolean isShowPoints() {
		return showPoints;
	}

	public void setShowPoints(boolean showPoints) {
		this.showPoints = showPoints;
	}
}
