package gui;

import geom.Shape;

import java.awt.event.MouseEvent;
import java.util.ArrayList;

import javax.swing.event.MouseInputAdapter;
import javax.vecmath.Matrix4d;
import javax.vecmath.Vector3d;
import javax.vecmath.Vector4d;

import core.Core;

import drawing.Catchable;
import drawing.Moveable;
import drawing.PaintBox;


public class MyInputAdapter extends MouseInputAdapter {
	
	private Catchable selected;
	private PaintBox paintBox;
	private Core core;
	private ArrayList<Shape> shapes;
	
	private Vector3d lastMousePoint;
	
	
	public MyInputAdapter(PaintBox paintBox, Core core) {
		
		this.core = core;
		this.paintBox = paintBox;
		this.shapes = paintBox.getShapes();
		
		paintBox.addMouseListener(this);
		paintBox.addMouseMotionListener(this);
	}

	public void mouseClicked(MouseEvent e) {
		
		if ( Core.instance.isPointEditing ) {
			
			if ( e.getButton() == MouseEvent.BUTTON1 ) {
				
				Vector3d mousePoint = computeMousePoint(e);
				Core.instance.addRoadPoint(mousePoint);
			}
			
			if ( e.getButton() == MouseEvent.BUTTON3 ) {
				
				Core.instance.deleteRoadPoint();
			}
		}
	}
	
	private Vector3d computeMousePoint(MouseEvent e) {
		
		Vector4d mousePoint = new Vector4d(e.getX(), e.getY(), 0, 1);

		Matrix4d invertWindow = new Matrix4d(paintBox.getWindowTransform());
		invertWindow.invert();
		invertWindow.transform(mousePoint);
		
		return new Vector3d(mousePoint.x / mousePoint.w, mousePoint.y / mousePoint.w, mousePoint.z / mousePoint.w);
	}

	public void mouseDragged(MouseEvent e) {
		
		Vector3d mousePoint = computeMousePoint(e);
		
		if ( this.lastMousePoint != null) {
			
			Vector3d delta = new Vector3d(mousePoint);
			delta.sub(lastMousePoint);
			
			//System.out.println(delta);
			
			if ( selected instanceof Moveable) {
				
				((Moveable)selected).move(delta.x, delta.y);
			}
		}
		
		this.lastMousePoint = mousePoint;
		this.paintBox.repaint();
	}

	public void mousePressed(MouseEvent e) {
		
		if ( e.getButton() == MouseEvent.BUTTON1) {

			Vector3d mousePoint = computeMousePoint(e);
			
			Catchable clicked = null;

			for (Shape shape : this.shapes) {

				if ( shape instanceof Catchable ) {
					Catchable cShape = (Catchable)shape;

					clicked = cShape.isCatch(mousePoint.x, mousePoint.y);
					
					if ( clicked != null) break;
				}
			}
			this.selected = clicked;
			this.paintBox.repaint();
			this.lastMousePoint = mousePoint;
		}
		
	}

	public void mouseReleased(MouseEvent e) {
		// TODO Auto-generated method stub
		super.mouseReleased(e);
		
		if ( this.selected != null && selected instanceof Moveable) {
			
			((Moveable)this.selected).afterMove();
		}
		
		this.paintBox.repaint();
	}

	public Catchable getSelected() {
		return selected;
	}
}
